/*
********************************************************************
* IBM Confidential                                                 *
*                                                                  *
* OCO Source Materials                                             *
*                                                                  *
* 5724-E76 and 5724-E77                                            *
*                                                                  *
* (C) Copyright IBM Corp. 2001, 2003                               *
*                                                                  *
* The source code for this program is not published or otherwise   *
* divested of its trade secrets, irrespective of what has been     *
* deposited with the U.S. Copyright Office.                        *
********************************************************************
*/
package com.ibm.wps.portlets.menu;

import org.apache.jetspeed.portlet.PortletAdapter;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.service.PortletServiceNotFoundException;
import org.apache.jetspeed.portlet.service.PortletServiceUnavailableException;

import com.ibm.wps.portlet.menu.MenuContext;
import com.ibm.wps.portlet.menu.MenuProvider;
import com.ibm.wps.portlet.menu.MenuTree;
import com.ibm.wps.portlet.menu.MenuTreeException;
import com.ibm.wps.portletservice.portletmenu.MemoryMenuService;

/**
 * {@link MemoryMenuTreePortlet} is a convenience implementation of a portlet that exposes a dynamic
 * (memory based) portlet menu. Custom subclasses of {@link MemoryMenuTreePortlet} need to
 * make sure that the menu is filled with appropriate dynamic information.
 * 
 * <P>The {@link MemoryMenuTreePortlet} is a default implementation for the
 * {@link org.apache.jetspeed.portlet.Portlet} interface. If a portlet uses an an in-memory menu tree,
 * it is recommended that it derives from {@link MemoryMenuTreePortlet}.
 * The controller interface {@link com.ibm.wps.portlet.menu.MenuTreeTopologyCtrl} enables adding or
 * removing tree nodes. The controller interface {@link com.ibm.wps.portlet.menu.MenuTreeInfoCtrl}
 * can be used to set title and description of the node and to add an URI action and one or
 * more parameters to the node.</P>
 * 
 * <P>The following example shows how {@link MenuProvider#getMenuTree} can be extended
 * to create a menu tree with two nodes and two subnodes of the first node. Title and description
 * are set in two different locales. To avoid that the memory tree is created at each request,
 * it is recommended to store the tree instance, e.g., as a session attribute.</P>
 * 
 * <P>Portlet source code:</P>
 * 
 * <PRE>
 * import org.apache.jetspeed.portlet.*;
 * import org.apache.jetspeed.portlet.event.*;
 * import com.ibm.wps.portlet.menu.*;
 * import com.ibm.wps.portlets.menu.*;
 * import java.util.*;
 * import java.io.*;
 * 
 * public class MemoryTestPortlet extends MemoryMenuTreePortlet
 * implements MenuProvider, ActionListener
 * {
 *     public void doView(PortletRequest request, PortletResponse response)
 *     throws PortletException, IOException
 *     {
 *         // Invoke the JSP to render
 *         getPortletConfig().getContext().include("/jsp/View.jsp", request, response);
 *     }
 * 
 * 
 *     // action event handling
 *     public void actionPerformed(ActionEvent event) throws PortletException
 *     {
 *         if (event != null)
 *         {
 *             String actionString = event.getActionString();
 * 
 *             PortletRequest request = event.getRequest();
 * 
 *             if (actionString != null)
 *             {
 *                 request.setAttribute("ActionString", actionString);
 *             }
 *         }
 *     }
 * 
 *     // overwrites implementation of MemoryMenuTreePortlet
 *     public MenuTree getMenuTree(MenuContext menuContext) throws PortletException
 *     {
 *         // session stores menu tree
 *         PortletSession session = menuContext.getPortletRequest().getPortletSession();
 * 
 *         MenuTree menuTree = (MenuTree) session.getAttribute("menu-tree");
 * 
 *         if ( menuTree == null )
 *         {
 *             // use method implementation of MemoryMenuTreePortlet
 *             menuTree = super.createEmptyMenuTree(menuContext);
 * 
 *             // get the controllers
 *             MenuTreeInfoCtrl    ctrlInfo      = (MenuTreeInfoCtrl) menuTree;
 *             MenuTreeTopologyCtrl ctrlTopology  = (MenuTreeTopologyCtrl) menuTree;
 * 
 *             try
 *             {
 *                 // get the root
 *                 MenuNode rootNode = menuTree.getRoot();
 * 
 *                 // set scope
 *                 ctrlTopology.setScope(rootNode,MenuNode.SCOPE_REQUEST);
 * 
 *                 // add root node title and description
 *                 ctrlInfo.setInformation(Locale.ENGLISH,"Memory Tree","Memory Tree",rootNode);
 *                 ctrlInfo.setInformation(Locale.FRENCH,"Memory Tree","Memory Tree",rootNode);
 * 
 *                 // add two new nodes with title and description
 *                 MenuNode node1 = ctrlTopology.addNode("id1",rootNode);
 *                 ctrlInfo.setInformation(Locale.ENGLISH,"Node 1","Node 1",node1);
 *                 ctrlInfo.setInformation(Locale.FRENCH,"Noeud 1","Noeud 1",node1);
 * 
 *                 MenuNode node2 = ctrlTopology.addNode("id2",rootNode);
 *                 ctrlInfo.setInformation(Locale.ENGLISH,"Node 2","Node 2",node2);
 *                 ctrlInfo.setInformation(Locale.FRENCH,"Noeud 2","Noeud 2",node2);
 * 
 *                 MenuNode node3 = ctrlTopology.addNode("id3",rootNode);
 *                 ctrlInfo.setInformation(Locale.ENGLISH,"Node 3","Node 3",node3);
 *                 ctrlInfo.setInformation(Locale.FRENCH,"Noeud 3","Noeud 3",node3);
 * 
 *                 // add two subnodes to node1 with title and description
 *                 MenuNode node1_1 = ctrlTopology.addNode("id1_1",node1);
 *                 ctrlInfo.setInformation(Locale.ENGLISH,"Node 1.1","Node 1.1",node1_1);
 *                 ctrlInfo.setInformation(Locale.FRENCH,"Noeud 1.1","Noeud 1.1",node1_1);
 * 
 *                 MenuNode node1_2 = ctrlTopology.addNode("id1_2",node1);
 *                 ctrlInfo.setInformation(Locale.ENGLISH,"Node 1.2","Node 1.2",node1_2);
 *                 ctrlInfo.setInformation(Locale.FRENCH,"Noeud 1.2","Noeud 1.2",node1_2);
 * 
 *                 // set action and add parameter-value pairs to node1
 *                 ctrlInfo.setAction("Node1Action",node1);
 *                 ctrlInfo.addActionParameter("Param1Name","Node1Param1Value",node1);
 *                 ctrlInfo.addActionParameter("Param2Name","Node1Param2Value",node1);
 * 
 *                 // set action and add parameter-value pairs to node2
 *                 ctrlInfo.setAction("Node2Action",node2);
 *                 ctrlInfo.addActionParameter("Param1Name","Node2Param1Value",node2);
 *                 ctrlInfo.addActionParameter("Param2Name","Node2Param2Value",node2);
 * 
 *                 // set url to node3
 *                 ctrlInfo.setURL("http://www.ibm.com/us/",node3);
 * 
 *             }
 *             catch ( MenuTreeException ex )
 *             {
 *                 throw new PortletException("Exception creating tree", ex);
 *             }
 * 
 *             session.setAttribute("menu-tree", menuTree);
 *         }
 * 
 *         return menuTree;
 *     }
 * }
 * </PRE>
 * 
 * <P>with <CODE>/jsp/View.jsp</CODE></P>
 * 
 * <PRE>
 * &lt;%@ page contentType="text/html"%&gt;
 * &lt;H3&gt;Test of MemoryMenuTree&lt;/H3&gt;
 * &lt;UL&gt;
 *  &lt;LI&gt;Expand root node and &lt;CITE&gt;Node 1&lt;/CITE&gt;.&lt;BR&gt;&lt;BR&gt;
 *  &lt;/LI&gt;
 *  &lt;LI&gt;Click &lt;CITE&gt;Node 1&lt;/CITE&gt; and &lt;CITE&gt;Node 2&lt;/CITE&gt;. Check if action string and request parameters are displayed.
 *      &lt;TABLE BORDER="0"&gt;
 *      &lt;TR&gt;
 *          &lt;TD&gt;Action:&lt;/TD&gt;
 *          &lt;TD&gt;&lt;%=request.getAttribute("ActionString")%&gt;&lt;/TD&gt;
 *      &lt;/TR&gt;&lt;TR&gt;
 *          &lt;TD&gt;Param1:&lt;/TD&gt;
 *          &lt;TD&gt;&lt;%=request.getParameter("Param1Name")%&gt;&lt;/TD&gt;
 *      &lt;/TR&gt;&lt;TR&gt;
 *          &lt;TD&gt;Param2:&lt;/TD&gt;
 *          &lt;TD&gt;&lt;%=request.getParameter("Param2Name")%&gt;&lt;/TD&gt;
 *      &lt;/TR&gt;
 *      &lt;/TABLE&gt;
 *      &lt;BR&gt;
 *  &lt;/LI&gt;
 *  &lt;LI&gt;Click &lt;CITE&gt;Node 3&lt;/CITE&gt;. Check if external html page is displayed and return to portal page.
 *  &lt;/LI&gt;
 * &lt;/UL&gt;
 * </PRE>
 * 
 * @ibm-spi
 * @since 5.0
 *  
 * @see PortletAdapter
 * @see MenuProvider
 * @see com.ibm.wps.portlet.menu.MenuTreeTopologyCtrl
 * @see com.ibm.wps.portlet.menu.MenuTreeInfoCtrl
 */
public abstract class MemoryMenuTreePortlet extends PortletAdapter
implements MenuProvider
{
    /** Copyright statement. */
    private static final String COPYRIGHT = com.ibm.wps.Copyright.SHORT;

    // This is a serializable class. All changes to this class need to be done 
    // on accordance to the Portal Policy for Serialization Compatability.
    // If you are not certain whether your changes to this class affect its 
    // serialization compatability, please consult any of the chief programmers.
    static final long serialVersionUID = 5100L;
    
    /**
     * ID of the root node
     */
    private static final String ROOT_ID = "RootId";

    /**
     * Constructor.
     * 
     * @example See above how to extend this method and add more nodes.
     */
    protected MemoryMenuTreePortlet()
    {
        super();
    }
    
    /**
     * Returns a memory menu tree with only one unnamed root node for a specified context,
     * overwrite with the custom portlet implementation.
     * 
     * As of release 5.0, this method can be implemented as well as <CODE>getMenuTree()</CODE>.
     * 
     * @param menuContext
     *               menu context contains the current request ID and response ID.
     * 
     * @return the memory menu tree with only the root node which has the node ID <CODE>RootId</CODE>.
     *         To get this ID, apply <CODE>menuTree.getRoot().getId()</CODE> to the returned <CODE>menuTree</CODE>.
     * 
     * @example See above how to extend this method and add more nodes.
     * 
     * @exception PortletException
     *                   if service not found or unavailable
     * @see MenuProvider
     */
    public MenuTree getMenu(MenuContext menuContext) throws PortletException 
    {
        MenuTree menuTree = getMenuTree(menuContext);

        return(menuTree);
    }

    /**
     * Returns a memory menu tree with only one unnamed root node for a specified context,
     * overwrite with the custom portlet implementation.
     * 
     * As of release 5.0, this method can be implemented as well as <CODE>getMenu()</CODE>.
     * 
     * @param menuContext
     *               contains the current request ID and response ID.
     * 
     * @return the memory menu tree with only the root node which has the node ID <CODE>RootId</CODE>.
     *         To get this ID, apply <CODE>menuTree.getRoot().getId()</CODE> to the returned <CODE>menuTree</CODE>.
     * 
     * @example See the example above how {@link MenuProvider#getMenu} was extended.
     * 
     * @exception PortletException
     *                   if service not found or unavailable
     */
    protected MenuTree getMenuTree(MenuContext menuContext) throws PortletException
    {
        MenuTree menuTree = createEmptyMenuTree(menuContext);

        return menuTree;
    }

    /**
     * Returns a memory menu tree with only one unnamed root node,
     * do not overwrite, but call this method from the derived class.
     * 
     * @param menuContext
     *               contains the current request ID and response ID.
     * 
     * @return the memory menu tree with only the root node which has the node ID <CODE>RootId</CODE>.
     *         To get this ID, apply <CODE>menuTree.getRoot().getId()</CODE> to the returned <CODE>menuTree</CODE>.
     * 
     * @example See above how to extend this method and add more nodes.
     * 
     * @exception PortletException
     *                   if service not found or unavailable
     */
    protected MenuTree createEmptyMenuTree(MenuContext menuContext) throws PortletException 
    {
        MenuTree menuTree = null;

        // context
        PortletContext portletContext = getPortletConfig().getContext();

        // service
        MemoryMenuService service=null;

        try
        {
            service = (MemoryMenuService) portletContext.getService(MemoryMenuService.class);
        }
        catch ( PortletServiceUnavailableException e )
        {
            throw new PortletException ("MemoryMenuTreePortlet: Exception: MemoryMenuService is unavailable");
        }
        catch ( PortletServiceNotFoundException e )
        {
            throw new PortletException ("MemoryMenuTreePortlet: Exception: MemoryMenuService has not been found");
        }

        // service for tree logging
        /* LogMenuService logService=null;
        
        try
        {
            logService = (LogMenuService) portletContext.getService(LogMenuService.class);
        }
        catch (PortletServiceUnavailableException e)
        {
            throw new PortletException ("MemoryMenuTreePortlet: Exception: The LogMenuService is unavailable");
        }
        catch (PortletServiceNotFoundException e)
        {
            throw new PortletException ("MemoryMenuTreePortlet: Exception: The LogMenuService has not been found");
        } */

        // get tree
        try
        {
            menuTree = service.getMenuTree(ROOT_ID, menuContext);

            // service for tree logging
            /* menuTree = logService.getMenuTree(menuTree); */
        }
        catch ( MenuTreeException mte )
        {
            throw new PortletException ("MemoryMenuTreePortlet: Exception getting menu tree from service");
        }

        return(menuTree);
    }    
}


