/*
********************************************************************
* IBM Confidential                                                 *
*                                                                  *
* OCO Source Materials                                             *
*                                                                  *
* 5724-E76 and 5724-E77                                            *
*                                                                  *
* (C) Copyright IBM Corp. 2001, 2003                               *
*                                                                  *
* The source code for this program is not published or otherwise   *
* divested of its trade secrets, irrespective of what has been     *
* deposited with the U.S. Copyright Office.                        *
********************************************************************
*/

package com.ibm.wps.portletservice.credentialvault;

import java.util.Map;
import java.util.List;
import java.util.Iterator;
import java.util.Locale;

import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.service.PortletService;
import org.apache.jetspeed.portlet.service.PortletServiceException;

import com.ibm.wps.portletservice.credentialvault.credentials.Credential;
import com.ibm.wps.services.credentialvault.VaultService;
import com.ibm.wps.sso.credentialvault.secrets.CredentialSecret;

import com.ibm.wps.util.ObjectID;

// JAAS classes
import javax.security.auth.Subject;

/**
 * Portlet Service for legacy portlets to access the WebSphere Portal Credential Vault.
 * The portlet service offers the following functionality:
 * <ul>
 * <li><b>Management of Credential Vault Slots.</b> Credential Vault Segments can be retrieved, and Vault Slots can be created and deleted.</li>
 * <li><b>Management of Credential Secrets.</b> The service allows to set or retrieve credentials from the credential vault.</li>
 * </ul>
 * <p>
 * Available <b>Secret Types</b> in the Credential Vault are definied as constants in this interface.
 * Please note that not all types have to be available in the defined Vault Segments, depending on the utilized Vault Adapter implementation.
 * Supported types per segment can be retrieved through {@link com.ibm.wps.portletservice.credentialvault.VaultSegmentConfig#getSupportedSecretTypes() VaultSegmentConfig#getSupportedSecretTypes()}.
 * </p>
 * <p>
 * Available <b>Credential Types</b> in the Credential Vault are defined externally in a credential type registry configuration file and can be retrieved
 * through the method {@link #getCredentialTypes() getCredentialTypes()} that returns their respective type string.
 * Dependent on the actual configuration, the following Credential Types can be available: 
 * </p>
 * <b><i>Credential Type String     = respective implementation class in package <code>com.ibm.wps.portletservice.credentialvault.credentials</code></i></b>
 * <br>
 * <ul>
 * <li>HttpBasicAuth       = HttpBasicAuthCredential</li>
 * <li>HttpFormBasedAuth   = HttpFormBasedAuthCredential</li>
 * <li>LtpaToken           = LtpaTokenCredential</li>
 * <li>SiteMinderToken     = SiteMinderTokenCredential</li>
 * <li>WebSealToken        = WebSealTokenCredential</li>
 * <li>JavaMail            = JavaMailCredential</li>
 * <li>JaasSubjectPassive  = JaasSubjectPassiveCredential</li>
 * <li>SimplePassive       = SimplePassiveCredential</li>
 * <li>UserPasswordPassive = UserPasswordPassiveCredential</li>
 * <li>BinaryPassive       = BinaryPassiveCredential</li>
 * </ul>
 *
 * @ibm-api
 * @since 4.1
 * @ibm-java2security
 **/
public interface CredentialVaultService extends PortletService
{
    /**
     * This is the name of a predefined slot to store the transient JAAS credential.
     */
    static String PREDEFINED_SLOT_USER_JAAS_SUBJECT =
        VaultService.PREDEFINED_SLOT_USER_JAAS_SUBJECT;

    /**
     * Used only for the super class CredentialSecret of the secret classes as secret type.
     * This type is not used for creating slots.
     **/
    int SECRET_TYPE_UNDEFINED = CredentialSecret.TYPE_UNDEFINED;
    /**
     * The user's secret is a userid/password pair.
     **/
    int SECRET_TYPE_USERID_STRING_PASSWORD_STRING =
        CredentialSecret.TYPE_USERID_STRING_PASSWORD_STRING;
    /**
     * The user's secret is in binary form.
     **/
    int SECRET_TYPE_BYTEARRAY = CredentialSecret.TYPE_BYTEARRAY;
    /**
     * The user's secret is a java object.
     **/
    int SECRET_TYPE_JAVA_OBJECT = CredentialSecret.TYPE_JAVA_OBJECT;
    /**
     * Type for credential objects that do not contain a secret. The credential only knows where to get the secret.
     **/
    int SECRET_TYPE_NO_SECRET_DATA = CredentialSecret.TYPE_NO_SECRET_DATA;
    /**
     * The user's secret is a JAAS Subject.
     **/
    int SECRET_TYPE_JAAS_SUBJECT = CredentialSecret.TYPE_JAAS_SUBJECT;

    /**
     *  Returns an Iterator over all available credential types that are registered in the credential type registry.
     *
     *  @return   Iterator over <code>String</code> objects representing all credential 
     *            types that are registered in the credential type registry
     **/
    Iterator getCredentialTypes();

    /**
     *  Returns a credential slot's description for the specified locale.
     *
     *  @param   slotId
     *           The credential (slot) id.
     *
     *  @param   locale
     *           The description locale. If set to <code>null</code>, the default locale will be used.
     *
     *  @return   The credential slot description for the specified locale.
     *  @exception   PortletServiceException
     *               if the description could not been retrieved.
     **/
    String getSlotDescription(String slotId, Locale locale)
        throws PortletServiceException;

    /**
     *  Returns an iterator of all credential slots that a portlet is authorized to use.
     *
     *  @param   request
     *           The portlet request is needed by the CredentialVault service in order to determine 
     *           information about the portlet.
     *
     *  @return   Iterator over <code>CredentialSlotConfig</code> objects representing 
     * 				all credential slots that a portlet is authorized to use
     *  @exception   PortletServiceException      
     *               if the list of slots could not been retrieved.
     **/
    Iterator getAccessibleSlots(PortletRequest request)
        throws PortletServiceException;

    /**
     *  Sets a credential's binary secret on a given slot.
     *
     *  @param   slotId
     *           The credential (slot) id.
     *
     *  @param   secret
     *           The credential secret data in binary form.
     *
     *  @param   portletRequest
     *           The portlet request is used to determine parameters like the portlet id and user id.
     *
     *  @exception   PortletServiceException
     *               if the credential secret is not of the type binary or if the secret could not be set.
     **/
    void setCredentialSecretBinary(
        String slotId,
        byte[] secret,
        PortletRequest portletRequest)
        throws PortletServiceException;

    /**
     *  Sets a credential's user/password secret on a given slot.
     *
     *  @param   slotId
     *           The credential (slot) id.
     *
     *  @param   userId
     *           The credential's userId.
     *
     *  @param   password
     *           The credential's password.
     *
     *  @param   portletRequest
     *           The portlet request is used to determine parameters like the portlet id and user id.
     *
     *  @exception   PortletServiceException
     *               if the credential secret is not of the type user/password or if the secret could not be set.
     **/
    void setCredentialSecretUserPassword(
        String slotId,
        String userId,
        char[] password,
        PortletRequest portletRequest)
        throws PortletServiceException;

    /**
     * Creates a new credential slot in the given vault segment.
     * This method should be used by "ordinary" portlets, but not by "admin portlets", because this method only
     * allows to create non system slots. The returned <code>CredentialSlotConfig</code> object holds the Slot ID
     * of the newly generated slot.
     * If one of the required parameters is null, an exception is thrown.
     *
     * @param resourceName
     *        Name of the resource. Must not be null.
     * @param segmentId
     *        ObjectId of the segment that this slot is created in. Must not be null.
     * @param descriptions
     *        The slot descriptions as a <code>Map</code>, keyed by their corresponding <code>Locale</code> objects
     *        Key:   <code>Locale</code> (max length 64 characters)
     *        Value: <code>String</code> (max length 255 characters)
     *        The given map can be empty.
     * @param keywords
     *        The slot keywords as a <code>Map</code>, keyed by their corresponding <code>Locale</code> objects
     *        Key:   <code>Locale</code> (max length 64 characters). Can be null.
     *        Value: <code>String</code> (max length 255 characters)
     *        The given map can be empty.
     * @param secretType
     *        The secrtet type identifier. Must not be null.
     * @param active
     *        Flag whether this credential may only be returned in form of an active credential object (<code>true</code>)
     *        or both as active and passive credential objects (<code>false</code>)
     * @param portletPrivate
     *        Flag whether the credential secret is portlet secific (<code>true</code>) 
     *        or shared between all of a user's portlets (<code>false</code>).
     * @param portletRequest
     *        The portlet request. Must not be null.
     * @return CredentialSlot
     *         The credential slot configuration as <code>CredentialSlotConfig</code> object.
     *  
     * @deprecated Use method {@link #createSlot(String, com.ibm.portal.ObjectID, Map, Map, int, boolean, boolean, PortletRequest)} taking a com.ibm.portal.ObjectID for the segmentId as parameter.    
     **/
    CredentialSlotConfig createSlot(
        String resourceName,
        ObjectID segmentId,
        Map descriptions,
        Map keywords,
        int secretType,
        boolean active,
        boolean portletPrivate,
        PortletRequest portletRequest)
        throws PortletServiceException;

    /**
     * Creates a new credential slot in the given vault segment.
     * This method should be used by "ordinary" portlets, but not by "admin portlets", because this method only
     * allows to create non system slots. The returned <code>CredentialSlotConfig</code> object holds the Slot ID
     * of the newly generated slot.
     * If one of the required parameters is null, an exception is thrown.
     *
     * @param resourceName
     *        Name of the resource. Must not be null.
     * @param segmentId
     *        ObjectId of the segment that this slot is created in. Must not be null.
     * @param descriptions
     *        The slot descriptions as a <code>Map</code>, keyed by their corresponding <code>Locale</code> objects
     *        Key:   <code>Locale</code> (max length 64 characters)
     *        Value: <code>String</code> (max length 255 characters)
     *        The given map can be empty.
     * @param keywords
     *        The slot keywords as a <code>Map</code>, keyed by their corresponding <code>Locale</code> objects
     *        Key:   <code>Locale</code> (max length 64 characters). Can be null.
     *        Value: <code>String</code> (max length 255 characters)
     *        The given map can be empty.
     * @param secretType
     *        The secrtet type identifier. Must not be null.
     * @param active
     *        Flag whether this credential may only be returned in form of an active credential object (<code>true</code>)
     *        or both as active and passive credential objects (<code>false</code>)
     * @param portletPrivate
     *        Flag whether the credential secret is portlet secific (<code>true</code>) 
     *        or shared between all of a user's portlets (<code>false</code>).
     * @param portletRequest 
     *        The portlet request. Must not be null.
     * @return CredentialSlot
     *         The credential slot configuration as <code>CredentialSlotConfig</code> object.
     **/
    CredentialSlotConfig createSlot(
        String resourceName,
        com.ibm.portal.ObjectID segmentId,
        Map descriptions,
        Map keywords,
        int secretType,
        boolean active,
        boolean portletPrivate,
        PortletRequest portletRequest)
        throws PortletServiceException;

    /**
     * Deletes an existing credential slot. The parameter is the Slot ID that can be retrieved through
     * {@link com.ibm.portal.portlet.service.credentialvault.CredentialSlotConfig#getSlotId() CredentialSlotConfig#getSlotId()}.
     *
     *  @param   slotName 
     *           The credential slot id.
     *  @exception   PortletServiceException
     *               Is thrown if the credential slot could not be deleted or found.
     **/
    public void deleteSlot(String slotName) throws PortletServiceException;

    /**
     * Returns the ObjectID of the default user managed vault segment.
     * Currently there is only one user-managed segment, so this returns the ID of <i>the</i> user managed segment.
     *
     * @return The ObjectID of the default user managed vault segment.
     * @exception   PortletServiceException
     *             Is thrown if the user segment could not be found.
     **/
    ObjectID getDefaultUserVaultSegmentId() throws PortletServiceException;

    /**
     * Returns a List of all available Vault Segments.
     *
     * @return List of <code>VaultSegmentConfig</code> objects representing all vault segments.
     * @exception   PortletServiceException
     *             Is thrown if the segments could not be retrieved.
     **/
    List getAllVaultSegments() throws PortletServiceException;

    /**
     *  Returns a specific credential from a specific slot. The type of the credential has to map the type of the stored credential.
     * The parameter is the Slot ID that can be retrieved through
     * {@link com.ibm.portal.portlet.service.credentialvault.CredentialSlotConfig#getSlotId() CredentialSlotConfig#getSlotId()}.
     * 
     *  @param   slotId
     *           The credential (slot) id.
     *
     *  @param   type
     *           The credential type as specified in the credential type registry.
     *
     *  @param   config
     *           The backend application specific configuration that is needed to initialize this credential. 
     *           This is one part of the credential configuration. The credential vault service will usually 
     *           add further information to this config from other sources: the user's secret from the actual credential store
     *           and the credential instance specific parameters from the portal's credential configuration.
     *           This parameter can be null.
     *
     *  @param   request
     *           The portlet request is used to determine information about the portlet.
     *
     *  @return   The retrieved credential object. The actual object will match the Credential Type.
     *
     *  @exception   PortletServiceException
     *               if the credential could not been retrieved - either for technical or secuity reasons.
     *
     *  @exception   CredentialSecretNotSetException
     *               if the requested credential secret is not set (by the user or admin).
     **/
    Credential getCredential(
        String slotId,
        String type,
        Map config,
        PortletRequest request)
        throws PortletServiceException, CredentialSecretNotSetException;

    /**
     *  Returns the user's JAAS Subject. The JAAS Subject is retrieved from the user object, so this method should only be called if a user is currently logged in. Otherwise a <code>PortletServiceException</code> is thrown. 
     *  Note: The returned object is a transient credential.
     * 
     *  @param   request
     *           The portlet request is used to determine the user.
     *
     *  @return   The user's JAAS Subject.
     *
     *  @exception   PortletServiceException
     *               if the subject could not been retrieved, e.g. because there is no logged in user.
     **/
    Subject getUserSubject(PortletRequest request)
        throws PortletServiceException;

}
