/*
********************************************************************
* IBM Confidential                                                 *
*                                                                  *
* OCO Source Materials                                             *
*                                                                  *
* 5724-E76 and 5724-E77                                            *
*                                                                  *
* (C) Copyright IBM Corp. 2001, 2003                               *
*                                                                  *
* The source code for this program is not published or otherwise   *
* divested of its trade secrets, irrespective of what has been     *
* deposited with the U.S. Copyright Office.                        *
********************************************************************
*/
package com.ibm.wps.portletservice.portletmenu;

import org.apache.jetspeed.portlet.service.PortletService;

import com.ibm.wps.portlet.menu.MenuContext;
import com.ibm.wps.portlet.menu.MenuTree;
import com.ibm.wps.portlet.menu.MenuTreeException;

/**
 * {@link MemoryMenuService} is a portlet service that allows the creation of a <b>dynamic</b> out-of-the-box portlet menu
 * implementation that greatly facilitates the implementation of the {@link com.ibm.wps.portlet.menu.MenuProvider} interface. 
 *  
 * This service creates an In-Memory tree that either has only one unnamed root node or incorporates 
 * an existing tree model, which can be both static or dynamic.</P>
 * 
 * A menu tree instance that is created by the {@link MemoryMenuService} also exposes the following
 * controller interfaces: {@link com.ibm.wps.portlet.menu.MenuTreeInfoCtrl}, {@link com.ibm.wps.portlet.menu.MenuTreeInfoCopyCtrl}, 
 * {@link com.ibm.wps.portlet.menu.MenuTreeTopologyCtrl} and {@link MemoryMenuTreeTopologyCtrl}.
 * 
 * <P>The following code shows how {@link com.ibm.wps.portlet.menu.MenuProvider#getMenu} can be implemented 
 * to return a {@link MenuTree} using 
 * {@link com.ibm.wps.portletservice.portletmenu.MemoryMenuService}.
 * <PRE>
 * public MenuTree getMenu(MenuContext menuContext) throws PortletException
 * {
 *     // get service
 *     MemoryMenuService service=null;
 * 
 *     try
 *     {
 *         service = (MemoryMenuService) portletContext.getService(MemoryMenuService.class);
 *     }
 *     catch ( PortletServiceUnavailableException e )
 *     {
 *         throw new PortletException ("MemoryMenuTreePortlet: Exception: MemoryMenuService is unavailable");
 *     }
 *     catch ( PortletServiceNotFoundException e )
 *     {
 *         throw new PortletException ("MemoryMenuTreePortlet: Exception: MemoryMenuService has not been found");
 *     }
 * 
 *     // get tree
 *     try
 *     {
 *         menuTree = service.getMenuTree("myRootId", contextOnLocalThread);
 * 
 *     }
 *     catch ( MenuTreeException mte )
 *     {
 *         throw new PortletException ("MemoryMenuTreePortlet: Exception getting menu tree from service");
 *     }
 * 
 *     return (menuTree);
 * }
 * </PRE></P>
 * 
 * @ibm-spi
 * @since 5.0
 */

public interface MemoryMenuService extends PortletService
{

    /**
     * Creates an empty in-memory tree with just one unnamed root element and identifier <CODE>rootID</CODE>.
     * Use the {@link com.ibm.wps.portlet.menu.MenuTreeTopologyCtrl} and
     * {@link com.ibm.wps.portlet.menu.MenuTreeInfoCtrl} to fill the tree with information.
     * 
     * @param rootID identifier of the root element
     * 
     * @param menuContext
     *               context to get access to the current {@link org.apache.jetspeed.portlet.PortletRequest} and
     *               {@link org.apache.jetspeed.portlet.PortletResponse}.
     * 
     * @return in-memory menu tree
     * 
     * @exception MenuTreeException
     *                   if the tree cannot be created
     */
    public MenuTree getMenuTree(String rootID,   MenuContext menuContext)  throws MenuTreeException;

    /**
     * Creates an in-memory copy of the portlet menu tree represented by <CODE>parent</CODE>.
     * If the <CODE>parent</CODE> tree topology contains cycles, e.g., a child node is parent of its grand-parent,
     * the nodes are only copied once.
     * 
     * @param parent menu tree to copy
     * 
     * @param menuContext
     *               context to get access {@link org.apache.jetspeed.portlet.PortletRequest} and
     *               {@link org.apache.jetspeed.portlet.PortletResponse}
     * 
     * @return in-memory tree that represents a copy of the original tree
     * 
     * @exception MenuTreeException
     *                   if the tree cannot be created
     * @see com.ibm.wps.portlet.menu.MenuTreeInfoCopyCtrl
     */
    public MenuTree getMenuTree(MenuTree parent, MenuContext menuContext)  throws MenuTreeException;
}
