/*
********************************************************************
* IBM Confidential                                                 *
*                                                                  *
* OCO Source Materials                                             *
*                                                                  *
* 5724-E76 and 5724-E77                                            *
*                                                                  *
* (C) Copyright IBM Corp. 2001, 2003                               *
*                                                                  *
* The source code for this program is not published or otherwise   *
* divested of its trade secrets, irrespective of what has been     *
* deposited with the U.S. Copyright Office.                        *
********************************************************************
*/
package com.ibm.wps.portletservice.portletmenu;

import java.io.Reader;

import org.apache.jetspeed.portlet.service.PortletService;

import com.ibm.wps.portlet.menu.MenuContext;
import com.ibm.wps.portlet.menu.MenuTree;

/**
 * {@link XMIMenuService} is a portlet service that allows the creation of a <b>static</b> out-of-the-box portlet menu
 * implementation that greatly facilitates the implementation of the {@link com.ibm.wps.portlet.menu.MenuProvider} interface. The content
 * of the static tree is based of a custom XMI file. 
 * 
 * This service creates a menu tree using XMI (EXtended Markup Interface).
 * The tree configuration is completely stored in an XML file which is accessed
 * through an {@link java.io.Reader} instance.</P>
 * 
 * A menu tree instance that is created by the {@link XMIMenuService} also exposes the following
 * controller interfaces: {@link com.ibm.wps.portlet.menu.MenuTreeInfoCopyCtrl}.
 * 
 * <P>The syntax of the XML file is described in the DTD file <CODE>menu&minus;tree.dtd</CODE>.
 * Once the portal navigation displayed a static tree on a page, it will not change until either the portlet
 * is deactivated, deleted from the page, or the corresponding portlet application is updated or re-installed.</P>
 * 
 * <P>The following code sample shows how {@link com.ibm.wps.portlet.menu.MenuProvider#getMenu} can be implemented
 * to return a {@link com.ibm.wps.portlet.menu.MenuTree} using
 * {@link com.ibm.wps.portletservice.portletmenu.MemoryMenuService}.
 * <PRE>
 * public MenuTree getMenu(MenuContext menuContext) throws PortletException
 * {
 *     // services
 *     XMIMenuService service = null;
 * 
 *     try
 *     {
 *         service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
 *     }
 *     catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
 *     {
 *         throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
 *     }
 *     catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
 *     {
 *         throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
 *     }
 * 
 *     // inputStreamReader reads XML file
 *     menuTree = service.getMenuTree(inputStreamReader, menuContext);
 * 
 *     return(menuTree);
 * }
 * </PRE>
 * </P>
 * 
 * @ibm-spi
 * @since 5.0
 */

public interface XMIMenuService extends PortletService
{
    /**
     * Creates a static menu tree from an XML file which is accessed through a {@link java.io.Reader}.
     * 
     * @param menuTreeStream
     *               reads the XML file with the tree configuration
     * 
     * @param menuContext
     *               context that stores request/response IDs
     * 
     * @return static XMI menu tree
     * 
     * @example </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * </PRE>                                                         <PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * </PRE>                                                         <PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * </PRE>                                                         <PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * </PRE>                                                         <PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * </PRE>                                                         <PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * </PRE>                                                         <PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * </PRE>                                                         <PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * </PRE>                                                         <PRE>
     * // services
     * XMIMenuService service = null;
     * 
     * try
     * {
     *     service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
     * }
     * catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
     * {
     *     throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
     * }
     * 
     * // inputStreamReader reads XML file
     * menuTree = service.getMenuTree(inputStreamReader, menuContext);
     * </PRE>
     * 
     */
    public MenuTree getMenuTree(Reader menuTreeStream, MenuContext menuContext);
}
